#include <stdbool.h>
#include <stdint.h>
#include <Adafruit_NeoPixel.h>
#include "Const.h"
#include "computer.h"

const uint8_t LED_PIN = 17;
Adafruit_NeoPixel strip = Adafruit_NeoPixel(GRID_WIDTH, LED_PIN, NEO_RBG + NEO_KHZ800);

uint32_t lastMillis = 0;

const uint_fast8_t BLINK_PIN = 13;
uint_fast8_t blink = 0;

// Crossfade
const bool DO_CROSSFADES = true;
uint8_t presetIndex = PRESET_COUNT - 1;
uint32_t animElapsed = 0;
uint32_t animDuration = 0;

void setup() {
	// USB to computer
	USB.begin(BAUD_RATE);

	pinMode(BLINK_PIN, OUTPUT);

	// Start dark
	strip.begin();
	strip.show();

	if (!DO_CROSSFADES) {
		fader = 1.0f;
	}

	computer_init(&strip);

	// Start default animation
	/*
	const char * anim = &DEFAULT_ANIMATION[0];
	while (*anim) {
		computer_input_from_usb(*anim++);
	}
	*/
}

void loop() {
	uint32_t now = millis();

	strip.show();

	// Crossfade
	if (DO_CROSSFADES) {
		// New animation now?
		if (fader < 0.0f) {
			presetIndex = (presetIndex + 1) % PRESET_COUNT;

			// Send the preset chars to the computer, until
			// we hit string terminator '\0'
			const char * str = PRESETS[presetIndex];
			while (*str) {
				computer_input_from_usb(*str);
				str++;
			}

			// Start crossfade
			computer_fade_in();
		}
	}

	// From Serial: From the laptop/programmer
	while (USB.available() > 0) {
		// read the incoming byte:
		uint8_t b = Serial.read();
		computer_input_from_usb(b);
	}

	uint8_t elapsed = now - lastMillis;
	lastMillis = now;

	computer_run(elapsed);

	// Blink every 64 updates, to prove we're alive.
	// If the device blinks faster than once per second, our frame rate is good!
	blink++;
	if (blink == 60) {
		blink = 0;
		digitalWrite(BLINK_PIN, HIGH);
	} else {
		digitalWrite(BLINK_PIN, LOW);
	}
}
