#ifndef LED_LAYOUT_H
#define LED_LAYOUT_H

#include <stdbool.h>
#include <stdint.h>
#include "Const.h"

#define TWOPI (6.283185f)

#define LEDS_ACROSS  (15)
#define LEDS_DOWN    (29)

void led_layout_set_all(float * x_ar, float * y_ar)
{
	const float SCALE = 1.0f / (LEDS_DOWN - 1);

	// Up the left side (outer strand)
	for (uint8_t i = 0; i < 14; i++) {
		x_ar[i] = 0;
		y_ar[i] = ((13 - i) * 2 + 1) * SCALE;
	}

	// Across the top (outer strand)
	for (uint8_t i = 14; i < 20; i++) {
		x_ar[i] = (i - 13) * 2 * SCALE;
		y_ar[i] = 0;
	}

	// Down the right side (outer strand)
	for (uint8_t i = 20; i < 34; i++) {
		x_ar[i] = 14 * SCALE;
		y_ar[i] = ((i - 20) * 2 + 1) * SCALE;
	}

	// Back up the right side (inner strand)
	for (uint8_t i = 34; i < 48; i++) {
		x_ar[i] = 14 * SCALE;
		y_ar[i] = ((48 - i) * 2) * SCALE;
	}

	// Back across the top (inner strand)
	for (uint8_t i = 48; i < 55; i++) {
		x_ar[i] = ((54 - i) * 2 + 1) * SCALE;
		y_ar[i] = 0;
	}

	// Down the left side (inner strand)
	for (uint8_t i = 55; i < 69; i++) {
		x_ar[i] = 0;
		y_ar[i] = ((i - 54) * 2) * SCALE;
	}

	// Snake across the bottom
	for (uint8_t i = 69; i < LED_COUNT; i++) {
		x_ar[i] = ((i - 69) * 2) * SCALE;
		if (i == (LED_COUNT - 1)) {
			x_ar[i] = 1.0f;
		}
		y_ar[i] = 1.0f;
	}
}

#endif
